@@ -6,13 +6,15 @@
 #define ALIGNER_1MM_H_
 
 #include <utility>
-#include <vector>
+
 #include "aligner.h"
+#include "ds.h"
 #include "hit.h"
 #include "range_source.h"
 #include "row_chaser.h"
 #include "range_chaser.h"
 #include "ref_aligner.h"
+#include "sstring.h"
 
 /**
  * Concrete factory class for constructing unpaired exact aligners.
@@ -20,11 +22,11 @@
 class Unpaired1mmAlignerV1Factory : public AlignerFactory {
 	typedef RangeSourceDriver<EbwtRangeSource> TRangeSrcDr;
 	typedef CostAwareRangeSourceDriver<EbwtRangeSource> TCostAwareRangeSrcDr;
-	typedef std::vector<TRangeSrcDr*> TRangeSrcDrPtrVec;
+	typedef EList<TRangeSrcDr*> TRangeSrcDrPtrVec;
 public:
 	Unpaired1mmAlignerV1Factory(
-			Ebwt<String<Dna> >& ebwtFw,
-			Ebwt<String<Dna> >* ebwtBw,
+			Ebwt& ebwtFw,
+			Ebwt* ebwtBw,
 			bool doFw,
 			bool doRc,
 			HitSink& sink,
@@ -33,8 +35,8 @@ public:
 			RangeCache *cacheBw,
 			uint32_t cacheLimit,
 			ChunkPool *pool,
-			BitPairReference* refs,
-			vector<String<Dna5> >& os,
+			BitPairReference *refs,
+			EList<BTRefString >& os,
 			bool maqPenalty,
 			bool qualOrder,
 			bool strandFix,
@@ -52,8 +54,7 @@ public:
 			cacheBw_(cacheBw),
 			cacheLimit_(cacheLimit),
 			pool_(pool),
-			refs_(refs),
-			os_(os),
+			os_(os), refs_(refs),
 			maqPenalty_(maqPenalty),
 			qualOrder_(qualOrder),
 			strandFix_(strandFix),
@@ -72,8 +73,8 @@ public:
 	virtual Aligner* create() const {
 
 		HitSinkPerThread* sinkPt = sinkPtFactory_.create();
-		EbwtSearchParams<String<Dna> >* params =
-			new EbwtSearchParams<String<Dna> >(*sinkPt, os_);
+		EbwtSearchParams* params =
+			new EbwtSearchParams(*sinkPt, os_);
 
 		const int halfAndHalf = 0;
 		const bool seeded = false;
@@ -140,8 +141,8 @@ public:
 		delete drVec;
 
 		// Set up a RangeChaser
-		RangeChaser<String<Dna> > *rchase =
-			new RangeChaser<String<Dna> >(cacheLimit_, cacheFw_, cacheBw_);
+		RangeChaser *rchase =
+			new RangeChaser(cacheLimit_, cacheFw_, cacheBw_);
 
 		// Set up the aligner
 		return new UnpairedAlignerV2<EbwtRangeSource>(
@@ -151,8 +152,8 @@ public:
 	}
 
 private:
-	Ebwt<String<Dna> >& ebwtFw_;
-	Ebwt<String<Dna> >* ebwtBw_;
+	Ebwt& ebwtFw_;
+	Ebwt* ebwtBw_;
 	bool doFw_;
 	bool doRc_;
 	HitSink& sink_;
@@ -161,8 +162,8 @@ private:
 	RangeCache *cacheBw_;
 	const uint32_t cacheLimit_;
 	ChunkPool *pool_;
-	BitPairReference* refs_;
-	vector<String<Dna5> >& os_;
+	EList<BTRefString >& os_;
+	BitPairReference *refs_;
 	const bool maqPenalty_;
 	const bool qualOrder_;
 	bool strandFix_;
@@ -177,12 +178,11 @@ private:
 class Paired1mmAlignerV1Factory : public AlignerFactory {
 	typedef RangeSourceDriver<EbwtRangeSource> TRangeSrcDr;
 	typedef CostAwareRangeSourceDriver<EbwtRangeSource> TCostAwareRangeSrcDr;
-	typedef std::vector<TRangeSrcDr*> TRangeSrcDrPtrVec;
+	typedef EList<TRangeSrcDr*> TRangeSrcDrPtrVec;
 public:
 	Paired1mmAlignerV1Factory(
-			Ebwt<String<Dna> >& ebwtFw,
-			Ebwt<String<Dna> >* ebwtBw,
-			bool color,
+			Ebwt& ebwtFw,
+			Ebwt* ebwtBw,
 			bool doFw,
 			bool doRc,
 			bool v1,
@@ -201,7 +201,7 @@ public:
 			uint32_t cacheLimit,
 			ChunkPool *pool,
 			BitPairReference* refs,
-			vector<String<Dna5> >& os,
+			EList<BTRefString >& os,
 			bool reportSe,
 			bool maqPenalty,
 			bool qualOrder,
@@ -212,7 +212,6 @@ public:
 			uint32_t seed) :
 			ebwtFw_(ebwtFw),
 			ebwtBw_(ebwtBw),
-			color_(color),
 			doFw_(doFw),
 			doRc_(doRc),
 			v1_(v1),
@@ -250,16 +249,16 @@ public:
 	virtual Aligner* create() const {
 		HitSinkPerThread* sinkPt = sinkPtFactory_.createMult(2);
 		HitSinkPerThread* sinkPtSe1 = NULL, * sinkPtSe2 = NULL;
-		EbwtSearchParams<String<Dna> >* params =
-			new EbwtSearchParams<String<Dna> >(*sinkPt, os_);
-		EbwtSearchParams<String<Dna> >* paramsSe1 = NULL, * paramsSe2 = NULL;
+		EbwtSearchParams* params =
+			new EbwtSearchParams(*sinkPt, os_);
+		EbwtSearchParams* paramsSe1 = NULL, * paramsSe2 = NULL;
 		if(reportSe_) {
 			sinkPtSe1 = sinkPtFactory_.create();
 			sinkPtSe2 = sinkPtFactory_.create();
 			paramsSe1 =
-				new EbwtSearchParams<String<Dna> >(*sinkPtSe1, os_);
+				new EbwtSearchParams(*sinkPtSe1, os_);
 			paramsSe2 =
-				new EbwtSearchParams<String<Dna> >(*sinkPtSe2, os_);
+				new EbwtSearchParams(*sinkPtSe2, os_);
 		}
 
 		const int halfAndHalf = 0;
@@ -415,12 +414,12 @@ public:
 			dr2RcVec->push_back(dr2Rc_Bw);
 		}
 
-		RefAligner<String<Dna5> >* refAligner =
-			new OneMMRefAligner<String<Dna5> >(color_, verbose_, quiet_);
+		RefAligner* refAligner =
+			new OneMMRefAligner(verbose_, quiet_);
 
 		// Set up a RangeChaser
-		RangeChaser<String<Dna> > *rchase =
-			new RangeChaser<String<Dna> >(cacheLimit_, cacheFw_, cacheBw_);
+		RangeChaser *rchase =
+			new RangeChaser(cacheLimit_, cacheFw_, cacheBw_);
 
 		if(v1_) {
 			PairedBWAlignerV1<EbwtRangeSource>* al = new PairedBWAlignerV1<EbwtRangeSource>(
@@ -456,9 +455,8 @@ public:
 	}
 
 private:
-	Ebwt<String<Dna> >& ebwtFw_;
-	Ebwt<String<Dna> >* ebwtBw_;
-	bool color_;
+	Ebwt& ebwtFw_;
+	Ebwt* ebwtBw_;
 	bool doFw_;
 	bool doRc_;
 	bool v1_;
@@ -477,7 +475,7 @@ private:
 	const uint32_t cacheLimit_;
 	ChunkPool *pool_;
 	BitPairReference* refs_;
-	vector<String<Dna5> >& os_;
+	EList<BTRefString >& os_;
 	const bool reportSe_;
 	const bool maqPenalty_;
 	const bool qualOrder_;
