@@ -44,17 +44,7 @@ fi
 # this variant creates static binaries with PIC
 source /hbb_exe_gc_hardened/activate
 
-mkdir /mybin
-echo  'res=`echo $@ | sed "s/-L.*$//"`; echo $res; echo $res; /opt/rh/devtoolset-7/root/usr/bin/ar $res;' > /mybin/ar
-chmod +x /mybin/ar && export PATH=/mybin:$PATH
-
-# make static-libs
-# if [ $? -ne 0 ] ; then
-#     echo "Unable to build tbb and/or zlib static dependencies"
-#     exit 1
-# fi
-
-make bowtie-bin.zip RELEASE_BUILD=1 EXTRA_FLAGS="--std=c++98"
+make bowtie-bin.zip
 if [ $? -ne 0 ] ; then
     echo "Unable to create bowtie package"
     exit 1
